This archive contains the distribution Stats-LikeR,
version 0.294:

  Get basic statistical functions, like in R, but with Perl using XS for performance

This software is copyright (c) 2026-present by David E. Condon.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.


This README file was generated by Dist::Zilla::Plugin::Readme v6.037.
